; Hooray for conditional code!
; Basically, if we're compiling for GUI,
; we leave the next line uncommented, if
; we want to make it independent, comment
; it out. It changes whether to compress
; graphics, how to display sprites, option
; box routines, etc.

#define GUI_COMPILE

#include	"ti83plus.inc"			;General TI-83 Plus include file
#include	"mirage.inc"			;MirageOS include file
#ifdef GUI_COMPILE
#include	"GUI.inc"			;GUI include file
#else
#define gcall(xxxx) call xxxx
 .addinstr B_CALL * EF 3 NOP 1
#endif
	.org	$9d93				;Origin (set back two to account for AsmPrgm)
	.db	$BB,$6D				;Compiled AsmPrgm token
	ret					;So TIOS wont run the program
	.db	1				;Identifier as MirageOS program
	.db	%00000000,%00000000
	.db	%00111100,%01111000
	.db	%00100010,%10000000
	.db	%00100010,%10000000
	.db	%00101100,%01110000
	.db	%00100010,%00001000
	.db	%00100010,%00001000
	.db	%00111100,%11110000
	.db	%00000000,%00000000
	.db	%00000000,%00000000
	.db	%00000011,%10000000
	.db	%00010111,%11010000
	.db	%00111111,%11111000
	.db	%00011111,%11110000
	.db	%00000000,%00000000
	.db	"Battleship",0			;Zero terminated description
						;Program code starts here

sram		= saferam2
array		= sram
difficulty	= sram+101
rotator		= sram+102
c1hits		= sram+103
p1hits		= sram+104
aistate		= sram+105
mecol		= sram+106
merow		= sram+107
initlast	= sram+107	;last initialized (to zero) variable
aicol		= sram+108
airow		= sram+109
aidir		= sram+110
players		= sram+111
startsp		= sram+112

RectFillPHeight	= 8660h

p1ship		= 0
p1hit		= 1
p1miss		= 2
c1ship		= 3
c1hit		= 4
c1miss		= 5

start:
	xor a			; Make sure link activity will work?
	out (0),a		; I don't really know if i need this... but it doesn't hurt to be safe.
	ld (startsp),sp		; allow fixing stack errors
	res textwrite,(iy+textflags)
linkfail:
#ifdef GUI_COMPILE
	call guiInit
	ret c
	ld ix,openingscreen
#else
	ld de,openingscreen
#endif
	call dispVRLE
	call fastcopy
	call waitkey
	cp skClear
	jp z,quittoshell	;after link failure, stack isn't right.
	ld hl,winPlayers
	gcall(guiOptionBox)
	ld (players),a
retry:
	ld sp,(startsp)		; stack errors are fixed
	ld hl,array
	ld de,array+1
	ld bc,initlast-array
	ld (hl),0
	ldir
	ld a,(players)
	or a
	jr nz,nodiff
	ld hl,winDiff
	gcall(guiOptionBox)
	ld (difficulty),a
nodiff:
	call placeships
	call randships
	ld a,1	;own ships
	call dispships
	call FastCopy
	ld b,50
	call delayb
	; Figure out which way the game will be played
	ld a,(players)
	or a
	jp z,OnePlayer
	bcall(_clrlcdf)
	ld hl,WaitStr
	ld de,0
	call setvputs
SynchronizationLoop:
	ld b,2
	call iRandom		; The randomness prevents symmetrical loops (which are unlikely in the first place).
	inc b			; reset z flag
	rra
	call c,getbytetios
	jr z,OtherCalcIsWaiting
	ei
	halt
	bcall(_getcsc)
	cp skClear
	jp z,quittoshell
	call sendbytetios
	jr nz,SynchronizationLoop
	; get board first, then send board
	call getbytetios
	jr nz,SynchronizationLoop
	cp 55
	jr nz,SynchronizationLoop
	call getboard
	call sendboard
	jr FirstPlayerStartGame
OtherCalcIsWaiting:
	; send board first, then get board
	ld a,55
	call sendbytetios
	jr nz,SynchronizationLoop
	call sendboard
	call getboard
TwoPlayerGameLoop:
	; recieve input from other calc
	xor a
	bcall(_clrlcdf)
	res textwrite,(iy+textflags)
	ld hl,WaitStr
	ld de,0
	call setvputs
sync_wait_loop:
	ei
	halt
	bcall(_getcsc)
	cp skClear
	jp z,quittoshell
	call getbytetios
	jr nz,sync_wait_loop
	push af			; row first
	call getbytetios
	jp nz,linkfail
	pop de
	ld e,a			; col second 
	call main_ai_shoot_here
FirstPlayerStartGame:
	call main_get_input
	; Send data to other calc
	ld a,(merow)		; row first
	call sendbytetios
	jp nz,linkfail
	ld a,(mecol)		; col second
	call sendbytetios
	jp nz,linkfail
	jr TwoPlayerGameLoop

OnePlayer:
	ld a,(difficulty)
	or a
	jp z,main_ai_easy	;med is dealt with later
main_ai_hard:
	call main_get_input
main_ai_hard_loop1:
	ld b,10			;randomly choose a square.
	call iRandom
	push af
	ld b,10
	call iRandom
	pop de
	ld e,a
	ld (aicol),de
	call main_ai_shoot_here	;shoot at it
	jr c,main_ai_hard_loop1
	jr z,main_ai_hard	;if we miss, don't encircle old point
	
main_ai_hard_gdir:
	call main_get_input
	ld c,20
main_ai_hard_loop2:
	dec c
	jr z,main_ai_hard_loop1	;likely no legit spots (20 tries) so guess randomly again
	push bc
	ld b,4
	call iRandom
	ld (aidir),a
	ld de,(aicol)
	call movedir
	push de
	call main_ai_shoot_here
	pop de
	pop bc
	jr z,main_ai_hard_gdir	;miss
	jr c,main_ai_hard_loop2
	ld (aicol),de

main_ai_hard_cont:
	call main_get_input
main_ai_hard_loop3:
	ld a,(aidir)
	ld de,(aicol)
	call movedir
	ld (aicol),de
	call main_ai_shoot_here
	jr c,main_ai_hard_reverse
	jr nz,main_ai_hard_cont
	call main_get_input

main_ai_hard_reverse:
	ld a,(aidir)
	xor 3
	ld (aidir),a
	ld de,(aicol)
	ld c,11
main_ai_reverse_loop:
	dec c
	jr z,main_ai_hard_loop1
	push af
	call movedir
	call getoffset
	pop af
	bit p1hit,(hl)
	jr nz,main_ai_reverse_loop
	bit p1miss,(hl)
	jr nz,main_ai_hard_loop1
main_ai_reverse_done:
	push de
	call main_ai_shoot_here
	pop de
	jp z,main_ai_hard	;miss
	ld (aicol),de
	jr main_ai_hard_cont

main_ai_easy:
	call main_get_input	;wait for player to go
main_ai_easy_loop:
	ld b,10			;randomly choose a square.
	call iRandom
	push af
	ld b,10
	call iRandom
	pop de
	ld e,a
	call main_ai_shoot_here	;shoot at it
	jr c,main_ai_easy_loop	;if we've already shot it, try another
	jr main_ai_easy		;repeat.

; Shoot at the location in de
; Carry = failed (already went there)
; Otherwise z = miss, nz = hit
; Uses difficulty, if hard checks if surrounded by misses
main_ai_shoot_here:
	call getoffset
	ld a,(difficulty)
	cp 2
	jr nz,main_ai_shoot_med
	push hl
	ld de,10
	inc hl
	bit p1miss,(hl)
	jr z,main_ai_shoot_not_all
	dec hl
	dec hl
	bit p1miss,(hl)
	jr z,main_ai_shoot_not_all
	inc hl
	or a
	sbc hl,de
	bit p1miss,(hl)
	jr z,main_ai_shoot_not_all
	add hl,de
	add hl,de
	bit p1miss,(hl)
	jr z,main_ai_shoot_not_all
	pop hl
	scf
	ret
main_ai_shoot_not_all:
	pop hl
main_ai_shoot_med:
	scf
	bit p1hit,(hl)
	ret nz		; carry = failed
	bit p1miss,(hl)
	ret nz
	bit p1ship,(hl)
	push af
	jr z,main_ai_miss
main_ai_hit:
	res p1ship,(hl)
	set p1hit,(hl)
	ld a,(p1hits)
	inc a
	cp 17
	jr z,YouLose
	ld (p1hits),a
	ld ix,hitsprite
	jr main_ai_done
main_ai_miss:
	set p1miss,(hl)
	ld a,(aistate)
	cp 1
	jr z,$+3
	xor a
	cp 2
	jr nz,$+4
	ld a,3
	ld (aistate),a
	ld ix,misssprite
main_ai_done:
	push ix
	call drawboard
	ld a,1
	call dispships
	pop ix
	call dispright
	call fastcopy
	ld b,184
	call mydelayb
	pop af
	ccf		;reset carry flag (we know it's set) but don't destroy z
	ret

YouWin:
	; if it's 2 player, tell the other one that they lost.
	ld a,(players)
	ld hl,winYouWin
	or a
	jr z,ShowWindow
	push hl
	ld a,(merow)		; row first
	call sendbytetios
	jp nz,linkfail
	ld a,(mecol)		; col second
	call sendbytetios
	jp nz,linkfail
	pop hl
ShowWindow:
	gcall(guiOptionBox)
	or a
	jp nz,quittoshell	;stack is messed up
	jp retry		;note that retry fixes the stack so we don't have to.
YouLose:
	ld hl,winYouLose
	jr ShowWindow

#ifdef GUI_COMPILE
#define GUI_MIN_VERSION GUI_VERSION_02
#include "loadgui.asm"
#endif

SendBoard:
	bcall(_clrlcdf)
	ld de,0
	ld hl,sendstr
	call setvputs
	ld b,100
	ld hl,array
sb_loop:
	push bc
	ld a,(hl)
	and 1<<p1ship
	push hl
sb_sync_loop:
	push af
	call sendbytetios
	pop bc
	ld a,b
	jr nz,sb_sync_loop
	pop hl
	inc hl
	pop bc
	djnz sb_loop
	ret

GetBoard:
	bcall(_clrlcdf)
	ld de,0
	ld hl,getstr
	call setvputs
	ld b,100
	ld hl,array
gb_loop:
	push bc
	push hl
gb_sync_loop:
	call getbytetios
	jr nz,gb_sync_loop
	pop hl
	res c1ship,(hl)		; Because I didn't bother removing randships.
	or a
	jr z,gb_noset
	set c1ship,(hl)
gb_noset:
	pop bc
	inc hl
	djnz gb_loop
	ret

main_get_input:
	ld de,(mecol)
main_redraw:
	push de
	call drawboard
	xor a
	call dispships  ;a=0
	pop de
main_loop:
	call disprot
	bcall(_getcsc)
	push af
	call fastcopys
	call disprot	;get rid of it
	ld a,(rotator)
	xor 1
	ld (rotator),a
	pop af
	cp sk2nd
	jr z,main_sel
	cp skEnter
	jr z,main_sel
	cp skClear
	jp z,quittoshell
	cp skGraph
	jr nz,main_notgraph
	push af
	push de
	call drawboard
	ld a,1
	call dispships
	call fastcopy
	ld b,184
	call mydelayb
	pop de
	pop af
	jr main_redraw
main_notgraph:
	cp 5
	jr nc,main_loop
	or a
	jr z,main_loop
	dec a
	call movedir
	jr main_redraw
main_sel:
	call getoffset
	bit c1hit,(hl)
	jp nz,main_loop
	bit c1miss,(hl)
	jp nz,main_loop
	ld (mecol),de
	bit c1ship,(hl)
	jr z,main_sel_miss
main_sel_hit:
	set c1hit,(hl)
	res c1ship,(hl)
	ld a,(c1hits)
	inc a
	cp 17
	jp z,YouWin
	ld (c1hits),a
	ld ix,hitsprite
	jr main_sel_done
main_sel_miss:
	set c1miss,(hl)
	ld ix,misssprite
main_sel_done:
	push ix
	call drawboard
	xor a
	call dispships
	pop ix
	call dispright
	call fastcopy
	ld b,184
	call mydelayb
	ret

movedir:
	or a	;skDown
	jr z,movedir_down
	dec a	;skLeft
	jr z,movedir_left
	dec a	;skRight
	jr z,movedir_right
movedir_up:
	ld a,d
	dec d
	or a
	ret nz
	ld d,a
	ret
movedir_down:
	ld a,d
	inc d
	cp 9
	ret nz
	ld d,a
	ret
movedir_right:
	ld a,e
	inc e
	cp 9
	ret nz
	ld e,a
	ret
movedir_left:
	ld a,e
	dec e
	or a
	ret nz
	ld e,a
	ret

disprot:
	push de
	call getupperlefttohl
	ld a,(rotator)
	ld b,a
	add a,a
	add a,a
	add a,b
	ld e,a
	ld d,0
	ld ix,firstrotsprite
	add ix,de
	ld b,5
	ld a,l
	ld l,h
	call iSprite
	pop de
	ret

; Have player place ships using keyboard input.
placeships:
	call drawboard
	ld b,5
	call placeship
	ld b,4
	call placeship
	ld b,3
	call placeship
	ld b,3
	call placeship
	ld b,2
placeship:
	ld a,1
	call dispships
	ld de,0		; current location
placeships_redraw:
	push de
	ld a,2
	ld hl,rectfillpheight
	ld (hl),a
	inc hl
	ld (hl),a
	inc hl
	ld (hl),a
	inc hl
	dec a
	ld (hl),a
	call getupperlefttohl
	call getlowerrighttode
	set plotLoc,(iy+plotFlags)
	call fastcopys
	push bc
	bcall(_fillrectpattern)
	pop bc
	pop de
placeships_kloop:
	call waitkey
	cp sk2nd
	jr z,placeships_sel
	cp skEnter
	jr z,placeships_sel
	cp skClear
	jp z,quittoshell
	cp skAlpha
	jr z,placeships_flip
	cp skDown
	jr z,placeships_down
	cp skUp
	jr z,placeships_up
	cp skRight
	jr z,placeships_right
	cp skLeft
	jr nz,placeships_kloop
placeships_left:
	ld a,e
	or a
	jr z,placeships_kloop
	dec e
	jr placeships_redraw
placeships_right:
	ld a,e
	inc a
	cp 10
	jr z,placeships_kloop
	bit 7,d
	jr z,placeships_righth
	ld e,a
	jr placeships_redraw
placeships_righth:
	add a,b
	cp 11
	jr z,placeships_kloop
	sub b
	ld e,a
	jr placeships_redraw
placeships_up:
	ld a,d
	and %01111111
	jr z,placeships_kloop
	dec d
	jr placeships_redraw
placeships_down:
	ld a,d
	res 7,a
	inc a
	cp 10
	jr z,placeships_kloop
	bit 7,d
	jr nz,placeships_downv
	ld d,a
	jr placeships_redraw
placeships_downv:
	add a,b
	cp 11
	jr z,placeships_kloop
	sub b
	set 7,a
	ld d,a
	jp placeships_redraw
placeships_flip:
	ld a,d
	xor 80h
	ld d,a
	bit 7,d
	jr nz,placeships_fliptov
	ld a,e
placeships_fliptov:
	res 7,a
	add a,b
	cp 11
	jp c,placeships_redraw
	ld a,10
	sub b
	bit 7,d
	jr z,placeships_fliptoh
	set 7,a
	ld d,a
	jp placeships_redraw
placeships_fliptoh:
	ld e,a
	jp placeships_redraw
placeships_sel:
	;check if there's something there
	push de
	ld c,b
placeships_selloop1:
	call getoffset
	bit p1ship,(hl)
	jr nz,placeships_no
	bit 7,d
	jr nz,placeships_selv
	inc e
	djnz placeships_selloop1
	jr placeships_skip1
placeships_selv:
	inc d
	djnz placeships_selloop1
placeships_skip1:
	pop de
	; nothing, so make it happen
	ld b,c
placeships_selloop2:
	call getoffset
	set p1ship,(hl)
	bit 7,d
	jr z,placeships_selh
	inc d
	djnz placeships_selloop2
	ret
placeships_selh:
	inc e
	djnz placeships_selloop2
	ret
placeships_no:
	pop de
	ld b,c
	jp placeships_redraw

randships:
	ld c,5
	call randship
	ld c,4
	call randship
	ld c,3
	call randship
	ld c,3
	call randship
	ld c,2
randship:
	ld b,2
	call iRandom
	or a
	jr nz,randships_vert
	ld b,10
	call iRandom
	ld d,a
	ld a,10
	sub c
	ld b,a
	call iRandom
	ld e,a
	jr randships_chk
randships_vert:
	ld b,10
	call iRandom
	ld e,a
	ld a,10
	sub c
	ld b,a
	call iRandom
	set 7,a
	ld d,a
randships_chk:
	;check if there's something there
	push de
	ld b,c
randships_selloop1:
	call getoffset
	bit c1ship,(hl)
	jr nz,randships_no
	bit 7,d
	jr nz,randships_selv
	inc e
	djnz randships_selloop1
	jr randships_skip1
randships_selv:
	inc d
	djnz randships_selloop1
randships_skip1:
	pop de
	; nothing, so make it happen
	ld b,c
randships_selloop2:
	call getoffset
	set c1ship,(hl)
	bit 7,d
	jr z,randships_selh
	inc d
	djnz randships_selloop2
	ret
randships_selh:
	inc e
	djnz randships_selloop2
	ret
randships_no:
	pop de
	jp randship	;try again
	
	
	

; Gets a pointer to square (e,d) in array
; destroys a
getoffset:
	push de
	ld a,d		; n
	add a,a		; *2
	add a,a		; *2
	add a,d		; +n
	add a,a		; *2 = 10n
	add a,e
	ld e,a
	ld d,0
	ld hl,array
	add hl,de
	pop de
	ret
	
getupperlefttohl:
	ld a,d
	add a,a
	add a,d
	add a,a
	add a,2
	ld h,a
	ld a,e
	add a,a
	add a,e
	add a,a
	add a,2
	ld l,a
	ret
getlowerrighttode:
	dec b
	ld a,d
	bit 7,d
	jr z,$+3
	add a,b
	add a,a
	add a,d
	bit 7,d
	jr z,$+3
	add a,b
	add a,a
	add a,6
	push af
	ld a,e
	bit 7,d
	jr nz,$+3
	add a,b
	add a,a
	add a,e
	bit 7,d
	jr nz,$+3
	add a,b
	inc b
	add a,a
	add a,6
	ld e,a
	pop af
	ld d,a
	ret

dispships:
	res plotLoc,(iy+plotFlags)
	push bc
	ld e,10
dispships_outer:
	dec e
	push af
	ld d,10
dispships_inner:
	dec d
	push af
	push de
	push af
	call getoffset
	pop af
	or a
	jr nz,dispships_own
	bit c1hit,(hl)
	jr z,dispships_nochit
	ld ix,graysprite
	call orSprite
	jr dispships_done
dispships_nochit:
	bit c1miss,(hl)
	jr z,dispships_done
	ld ix,plussprite
	call orSprite
	jr dispships_done
dispships_own:
	bit p1ship,(hl)
	jr z,dispships_nopship
	ld ix,solidsprite
	call orSprite
	jr dispships_done
dispships_nopship:
	bit p1hit,(hl)
	jr z,dispships_nophit
	ld ix,graysprite
	call orSprite
	jr dispships_done
dispships_nophit:
	bit p1miss,(hl)
	jr z,dispships_done
	ld ix,plussprite
	call orSprite
dispships_done:
	pop de
	pop af
	jr nz,dispships_inner
	pop af
	jr nz,dispships_outer
	pop bc
	ret

; Draws the 10x10 grid
drawboard:
	bcall(_clrgbuf)
	ld hl,1*256+1
	ld de,61*256+1
	ld bc,6
	ld a,11
drawboard_loop1:
	call fastlinebs
	add hl,bc
	ex de,hl
	add hl,bc
	dec a
	jr nz,drawboard_loop1
	ld hl,1*256+1
	ld de,1*256+61
	ld bc,6*256
	ld a,11
drawboard_loop2:
	call fastlinebs
	add hl,bc
	ex de,hl
	add hl,bc
	dec a
	jr nz,drawboard_loop2
	ld ix,rightside
dispsprite:
	ld bc,58*256+3
#ifdef GUI_COMPILE
	ld a,BM_Xor+BM_VRLE
	ld de,2*256+67
	gcall(guiDrawSprite)
#else
	ld a,67
	ld l,2
	call iLSprite
#endif
	ret

dispright:
	ld hl,plotsscreen+8
	ld de,9
	xor a
	ld b,64
dispright_loop:
	ld (hl),a
	inc hl
	ld (hl),a
	inc hl
	ld (hl),a
	inc hl
	ld (hl),a
	add hl,de
	djnz dispright_loop
	jr dispsprite

; Fastlineb but saves registers
fastlinebs:
	push hl
	push de
	push bc
	push af
	call fastlineb
	pop af
	pop bc
	pop de
	pop hl
	ret

waitkey:
	ei
	halt
	bcall(_getcsc)
	or a
	jr z,waitkey
	ret

mydelayb:
	push hl
	push af
	ei
mydelay_loop:
	halt
	bcall(_getcsc)
	or a
	jr nz,mydelay_done
	djnz mydelay_loop
mydelay_done:
	cp skClear
	jp z,quittoshell
	pop af
	pop hl
	ret

; In: de -> graphic
#ifdef GUI_COMPILE
dispVRLE:
	ld de,0
	ld bc,64*256+12
	ld a,BM_Or+BM_VRLE
	gcall(guiDrawSprite)
	ret
#else
dispVRLE:				; Total: 48 bytes
	di				; 1
	xor a				; 1
	ex af,af'			; 1
	ld hl,plotsscreen		; 3
	ld c,12				; 2
dispVRLE_outer:
	push hl				; 1
	ld b,64				; 2
dispVRLE_inner:
	ld a,(de)			; 1
	inc de				; 1
	cp 91h				; 2
	jr nz,dispVRLE_norun		; 2
	ld a,(de)			; 1
	inc de				; 1
	ex af,af'			; 1
	or a				; 1
	jr nz,dispVRLE_inrun		; 2
	ld a,(de)			; 1
dispVRLE_inrun:
	dec a				; 1
	jr nz,dispVRLE_repeat		; 2
	inc de				; 1
	jr dispVRLE_rundone		; 2
dispVRLE_repeat:
	dec de				; 1
	dec de				; 1
dispVRLE_rundone:
	ex af,af'			; 1
dispVRLE_norun:
	ld (hl),a			; 1
	push de				; 1
	ld de,12			; 3
	add hl,de			; 1
	pop de				; 1
	djnz dispVRLE_inner		; 2
	pop hl				; 1
	inc hl				; 1
	dec c				; 1
	jr nz,dispVRLE_outer		; 2
	ret				; 1
#endif

#ifdef GUI_COMPILE
orsprite:
	call getupperlefttohl
	ex de,hl
	ld bc,5*256+1
	ld a,BM_Or
	gcall(guiDrawSprite)
	ret
#else
orSprite:
	call getupperlefttohl
	ld b,5
	ld a,l
	ld l,h
	ld	e,l		; copy of putSprite by Joe W.
	ld	h,$00
	ld	d,h
	add	hl,de
	add	hl,de
	add	hl,hl
	add	hl,hl
	ld	e,a
	and	$07
	ld	c,a
	srl	e
	srl	e
	srl	e
	add	hl,de
	ld	de,gbuf
	add	hl,de
orSpriteLoop1:
	ld	d,(ix)
	ld	e,$00
	ld	a,c
	or	a
	jr	z,orSpriteSkip1
orSpriteLoop2:
	srl	d
	rr	e
	dec	a
	jr	nz,orSpriteLoop2
orSpriteSkip1:
	ld	a,(hl)
	or	d		; used to be xor d
	ld	(hl),a
	inc	hl
	ld	a,(hl)
	or	e		; sed to be xor e
	ld	(hl),a
	ld	de,$0B
	add	hl,de
	inc	ix
	djnz	orSpriteLoop1
	ret
#endif

#ifndef GUI_COMPILE
;Slightly modified for this prog
guiOptionBox:
 res textwrite,(iy+sgrflags)
 set plotLoc,(iy+plotFlags)
 res fracDrawLFont,(iy+fontFlags)
 push hl
 ld hl,13*256+22
 ld de,47*256+73
 B_CALL _DrawRectBorderC
 ld hl,14*256+23
 ld de,19*256+72
 B_CALL _FillRect
 pop hl
 set textinverse,(iy+textflags)
 ld a,13
 push hl
 call centertext
 pop hl
 call nextstr
 res textinverse,(iy+textflags)
 ld de,20*256+24
 ld (pencol),de
 bcall(_vputs)
 ld de,26*256+24
 ld (pencol),de
 bcall(_vputs)
 ld de,32*256+24
 ld (pencol),de
 bcall(_vputs)
 ld e,0
guiOptionBoxRedraw:
 ld d,0
 push hl
 ld hl,39*256+24
 ld (pencol),hl
 pop hl
 set texterasebelow,(iy+textflags)
 ld b,(hl)
 push hl
 push bc
 inc hl
guiOptionBoxDrawLoop:
 ld a,d
 cp e
 jr nz,guiOptionBoxNotInverse
 set textinverse,(iy+textflags)
guiOptionBoxNotInverse:
 B_CALL _VPutS
 res textinverse,(iy+textflags)
 push de
 push hl
 ld a,lspace
 B_CALL _VPutMap
 pop hl
 pop de
 inc d
 djnz guiOptionBoxDrawLoop
 pop bc
 pop hl
guiOptionBoxWaitkeyLoop:
 ei
 halt
 push hl
 B_CALL _getcsc
 pop hl
 cp sk2nd
 jr z,guiOptionBoxSelect
 cp skEnter
 jr z,guiOptionBoxSelect
 cp skClear
 jp z,quittoshell
 sub 2
 jr z,guiOptionBoxLeft
 dec a
 jr nz,guiOptionBoxWaitkeyLoop
guiOptionBoxRight:
 ld a,e
 inc a
 cp b
 jr z,guiOptionBoxWaitkeyLoop
 ld e,a
 jr guiOptionBoxRedraw
guiOptionBoxLeft:
 ld a,e
 or a
 jr z,guiOptionBoxWaitkeyLoop
 dec a
 ld e,a
 jr guiOptionBoxRedraw
guiOptionBoxSelect:
 ld a,e
 ret
#endif


openingscreen:
	.db	$00,$00,$00,$7F,$7F,$73,$73,$73,$7F,$7F,$7F,$73
	.db	$73,$73,$7F,$7F,$91,$00,$17,$1E,$01,$01,$07,$04
	.db	$02,$02,$01,$01,$91,$00,$13,$01,$81,$83,$83,$87
	.db	$06,$8E,$8E,$9F,$9F,$BC,$BC,$38,$91,$00,$17,$E0
	.db	$B0,$10,$FF,$00,$02,$00,$00,$FF,$91,$00,$13,$83
	.db	$83,$C3,$C0,$E0,$60,$70,$70,$F8,$F8,$3C,$3C,$1C
	.db	$91,$00,$18,$7C,$44,$FF,$00,$22,$00,$00,$FF,$91
	.db	$00,$13,$FD,$FD,$FD,$91,$F0,$0A,$91,$00,$0B,$01
	.db	$02,$04,$04,$08,$08,$10,$10,$10,$20,$20,$20,$21
	.db	$FF,$21,$FF,$20,$22,$10,$10,$FF,$08,$08,$04,$04
	.db	$02,$01,$91,$00,$0D,$FF,$FF,$FF,$91,$78,$0A,$91
	.db	$00,$05,$01,$06,$18,$20,$40,$80,$91,$00,$08,$02
	.db	$0F,$08,$08,$0A,$FF,$0A,$FF,$00,$22,$00,$00,$FF
	.db	$91,$00,$06,$80,$40,$20,$18,$06,$01,$91,$00,$07
	.db	$91,$78,$0A,$7F,$7F,$7F,$00,$00,$02,$02,$3F,$C2
	.db	$02,$02,$02,$07,$91,$02,$05,$07,$02,$02,$02,$4A
	.db	$FF,$02,$02,$02,$FF,$02,$FF,$02,$22,$02,$02,$FF
	.db	$02,$07,$91,$02,$05,$07,$02,$02,$02,$C2,$3F,$02
	.db	$02,$91,$00,$04,$7F,$7F,$7F,$78,$78,$7F,$7F,$7F
	.db	$78,$78,$7F,$7F,$7F,$91,$00,$04,$E0,$1C,$03,$91
	.db	$00,$0D,$FF,$AB,$01,$05,$FF,$05,$FF,$00,$22,$00
	.db	$00,$FF,$91,$00,$0A,$03,$1C,$E0,$91,$00,$06,$0F
	.db	$1F,$3F,$3C,$3C,$3F,$3F,$1F,$03,$03,$3F,$3F,$1F
	.db	$91,$00,$07,$C0,$20,$10,$08,$04,$02,$01,$01,$91
	.db	$00,$08,$04,$FF,$04,$FF,$00,$22,$00,$00,$FF,$00
	.db	$00,$01,$01,$02,$04,$08,$10,$20,$C0,$91,$00,$09
	.db	$8F,$CF,$CF,$0F,$0F,$8F,$91,$CF,$05,$8F,$0F,$91
	.db	$00,$0F,$80,$80,$40,$40,$40,$91,$20,$04,$F9,$21
	.db	$FF,$20,$22,$40,$40,$FF,$80,$80,$91,$00,$11,$91
	.db	$1E,$05,$FE,$FE,$FE,$91,$1E,$05,$91,$00,$18,$F0
	.db	$10,$FF,$00,$22,$00,$00,$FF,$91,$00,$13,$91,$79
	.db	$0D,$91,$00,$17,$0E,$1B,$11,$FF,$00,$20,$00,$00
	.db	$FF,$91,$00,$13,$FC,$FE,$FE,$CE,$CE,$CE,$FE,$FE
	.db	$FC,$91,$C0,$04,$91,$00,$17,$F0,$00,$00,$E0,$20
	.db	$40,$40,$80,$80,$91,$00,$10


#ifdef GUI_COMPILE
; We can use VRLE compressed sprites
hitsprite:
	.db	%00000000,%10010001,%00000011,%00001111,%10010001,%00000000,%00000101,%10010001,%00000011,%00000100,%10010001,%00000000
	.db	%00001000,%10010001,%00000011,%00000100,%10010001,%00000000,%00000101,%10010001,%00000011,%00000100,%10010001,%00000000
	.db	%00001101,%10010001,%11000011,%00000101,%10010001,%11111111,%00000101,%10010001,%11000011,%00000101,%10010001,%00000000
	.db	%00000101,%10010001,%11111111,%00000100,%10010001,%00111100,%00001000,%10010001,%11111111,%00000100,%10010001,%00000000
	.db	%00000101,%10010001,%11111111,%00000100,%10010001,%00111100,%00001100,%00000000,%10010001,%11000000,%00001111,%10010001
	.db	%00000000,%00000101,%10010001,%11000000,%00000100,%10010001,%00000000,%00001000,%10010001,%11000000,%00000100,%10010001
	.db	%00000000,%00000101,%10010001,%11000000,%00000100,%10010001,%00000000,%00001100
misssprite:
	.db	%10010001,%00000111,%00001101,%00000000,%00000000,%00000111,%00000111,%00000111,%10010001,%00000000,%00000111,%00000111
	.db	%00000111,%00000111,%00000000,%00000000,%00000011,%10010001,%00000111,%00000101,%00000011,%00000001,%00000000,%00000000
	.db	%00000111,%00000111,%00000011,%00000000,%00000000,%00000011,%10010001,%00000111,%00000101,%00000011,%00000001,%00000000
	.db	%00000000,%00000111,%00000111,%00000011,%00000001,%10000011,%11000111,%11101111,%11111111,%01111101,%00111001,%00010001
	.db	%10010001,%00000001,%00000101,%00000000,%00000000,%11111111,%11111111,%11111111,%10010001,%00111000,%00000111,%11111111
	.db	%11111111,%11111111,%00000000,%00000000,%11111111,%11111111,%11111111,%00000000,%00000000,%11111111,%11111111,%11111111
	.db	%00000001,%00000001,%11111111,%11111111,%11111111,%00000000,%00000000,%11111111,%11111111,%11111111,%00000000,%00000000
	.db	%11111111,%11111111,%11111111,%00000001,%00000001,%11111111,%11111111,%11111111,%10010001,%11000000,%00001101,%00000000
	.db	%00000000,%11000000,%11000000,%11000000,%10010001,%00000000,%00000111,%11000000,%11000000,%11000000,%00000000,%00000000
	.db	%10000000,%11000000,%11000000,%00000000,%00000000,%00000000,%10000000,%10010001,%11000000,%00000101,%10000000,%00000000
	.db	%00000000,%10000000,%11000000,%11000000,%00000000,%00000000,%00000000,%10000000,%10010001,%11000000,%00000101,%10000000
rightside:
	.db	%00111110,%00111111,%00110011,%00111110,%00111111,%00110011,%00111111,%00111110,%00000000,%00000000,%00001000,%00011100
	.db	%00111110,%00110110,%01110111,%01111111,%11100011,%11000001,%00000000,%00000000,%01111111,%01111111,%10010001,%00011100
	.db	%00000110,%00000000,%00000000,%01111111,%01111111,%10010001,%00011100,%00000110,%00000000,%00000000,%10010001,%00111000
	.db	%00000110,%00111110,%00111110,%00000000,%00000000,%00111110,%00111110,%00111000,%00111110,%00111110,%00111000,%00111110
	.db	%00111110,%10010001,%00000000,%00010000,%10000000,%10000000,%10010001,%00000001,%00001000,%10010001,%00000000,%00100110
	.db	%01111000,%11111100,%11000000,%11111000,%01111100,%00001100,%11111100,%01111000,%10010001,%00000000,%00000100,%11001110
	.db	%11001110,%11001110,%11111110,%11111110,%11001110,%11001110,%11001110,%10010001,%00000000,%00000101,%10010001,%01110000
	.db	%00001000,%10010001,%00000000,%00000100,%11111000,%11111100,%11001100,%11001100,%11111100,%11111000,%11000000,%11000000
	.db	%10010001,%00000000,%00000111
#else
; Non-compressed versions
rightside:
	.db	%00111110,%00000000,%00000000
	.db	%00111111,%00000000,%00000000
	.db	%00110011,%00000000,%00000000
	.db	%00111110,%00000000,%00000000
	.db	%00111111,%00000000,%00000000
	.db	%00110011,%00000000,%00000000
	.db	%00111111,%00000000,%01111000
	.db	%00111110,%00000000,%11111100
	.db	%00000000,%00000000,%11000000
	.db	%00000000,%00000000,%11111000
	.db	%00001000,%00000000,%01111100
	.db	%00011100,%00000000,%00001100
	.db	%00111110,%00000000,%11111100
	.db	%00110110,%00000000,%01111000
	.db	%01110111,%00000000,%00000000
	.db	%01111111,%00000000,%00000000
	.db	%11100011,%10000000,%00000000
	.db	%11000001,%10000000,%00000000
	.db	%00000000,%00000001,%11001110
	.db	%00000000,%00000001,%11001110
	.db	%01111111,%00000001,%11001110
	.db	%01111111,%00000001,%11111110
	.db	%00011100,%00000001,%11111110
	.db	%00011100,%00000001,%11001110
	.db	%00011100,%00000001,%11001110
	.db	%00011100,%00000001,%11001110
	.db	%00011100,%00000000,%00000000
	.db	%00011100,%00000000,%00000000
	.db	%00000000,%00000000,%00000000
	.db	%00000000,%00000000,%00000000
	.db	%01111111,%00000000,%00000000
	.db	%01111111,%00000000,%01110000
	.db	%00011100,%00000000,%01110000
	.db	%00011100,%00000000,%01110000
	.db	%00011100,%00000000,%01110000
	.db	%00011100,%00000000,%01110000
	.db	%00011100,%00000000,%01110000
	.db	%00011100,%00000000,%01110000
	.db	%00000000,%00000000,%01110000
	.db	%00000000,%00000000,%00000000
	.db	%00111000,%00000000,%00000000
	.db	%00111000,%00000000,%00000000
	.db	%00111000,%00000000,%00000000
	.db	%00111000,%00000000,%11111000
	.db	%00111000,%00000000,%11111100
	.db	%00111000,%00000000,%11001100
	.db	%00111110,%00000000,%11001100
	.db	%00111110,%00000000,%11111100
	.db	%00000000,%00000000,%11111000
	.db	%00000000,%00000000,%11000000
	.db	%00111110,%00000000,%11000000
	.db	%00111110,%00000000,%00000000
	.db	%00111000,%00000000,%00000000
	.db	%00111110,%00000000,%00000000
	.db	%00111110,%00000000,%00000000
	.db	%00111000,%00000000,%00000000
	.db	%00111110,%00000000,%00000000
	.db	%00111110,%00000000,%00000000

hitsprite:
	.db	%00000000,%00000000,%00000000
	.db	%00000011,%11000011,%11000000
	.db	%00000011,%11000011,%11000000
	.db	%00000011,%11000011,%11000000
	.db	%00000011,%11000011,%11000000
	.db	%00000011,%11000011,%11000000
	.db	%00000011,%11111111,%11000000
	.db	%00000011,%11111111,%11000000
	.db	%00000011,%11111111,%11000000
	.db	%00000011,%11111111,%11000000
	.db	%00000011,%11111111,%11000000
	.db	%00000011,%11000011,%11000000
	.db	%00000011,%11000011,%11000000
	.db	%00000011,%11000011,%11000000
	.db	%00000011,%11000011,%11000000
	.db	%00000011,%11000011,%11000000
	.db	%00000000,%00000000,%00000000
	.db	%00000000,%00000000,%00000000
	.db	%00000000,%00000000,%00000000
	.db	%00000000,%00000000,%00000000
	.db	%00000000,%00000000,%00000000
	.db	%00000011,%11111111,%11000000
	.db	%00000011,%11111111,%11000000
	.db	%00000011,%11111111,%11000000
	.db	%00000011,%11111111,%11000000
	.db	%00000000,%00111100,%00000000
	.db	%00000000,%00111100,%00000000
	.db	%00000000,%00111100,%00000000
	.db	%00000000,%00111100,%00000000
	.db	%00000000,%00111100,%00000000
	.db	%00000000,%00111100,%00000000
	.db	%00000000,%00111100,%00000000
	.db	%00000000,%00111100,%00000000
	.db	%00000011,%11111111,%11000000
	.db	%00000011,%11111111,%11000000
	.db	%00000011,%11111111,%11000000
	.db	%00000011,%11111111,%11000000
	.db	%00000000,%00000000,%00000000
	.db	%00000000,%00000000,%00000000
	.db	%00000000,%00000000,%00000000
	.db	%00000000,%00000000,%00000000
	.db	%00000000,%00000000,%00000000
	.db	%00000011,%11111111,%11000000
	.db	%00000011,%11111111,%11000000
	.db	%00000011,%11111111,%11000000
	.db	%00000011,%11111111,%11000000
	.db	%00000000,%00111100,%00000000
	.db	%00000000,%00111100,%00000000
	.db	%00000000,%00111100,%00000000
	.db	%00000000,%00111100,%00000000
	.db	%00000000,%00111100,%00000000
	.db	%00000000,%00111100,%00000000
	.db	%00000000,%00111100,%00000000
	.db	%00000000,%00111100,%00000000
	.db	%00000000,%00111100,%00000000
	.db	%00000000,%00111100,%00000000
	.db	%00000000,%00111100,%00000000
	.db	%00000000,%00111100,%00000000

misssprite:
	.db	%00000111,%00000001,%11000000
	.db	%00000111,%10000011,%11000000
	.db	%00000111,%11000111,%11000000
	.db	%00000111,%11101111,%11000000
	.db	%00000111,%11111111,%11000000
	.db	%00000111,%01111101,%11000000
	.db	%00000111,%00111001,%11000000
	.db	%00000111,%00010001,%11000000
	.db	%00000111,%00000001,%11000000
	.db	%00000111,%00000001,%11000000
	.db	%00000111,%00000001,%11000000
	.db	%00000111,%00000001,%11000000
	.db	%00000111,%00000001,%11000000
	.db	%00000000,%00000000,%00000000
	.db	%00000000,%00000000,%00000000
	.db	%00000111,%11111111,%11000000
	.db	%00000111,%11111111,%11000000
	.db	%00000111,%11111111,%11000000
	.db	%00000000,%00111000,%00000000
	.db	%00000000,%00111000,%00000000
	.db	%00000000,%00111000,%00000000
	.db	%00000000,%00111000,%00000000
	.db	%00000000,%00111000,%00000000
	.db	%00000000,%00111000,%00000000
	.db	%00000000,%00111000,%00000000
	.db	%00000111,%11111111,%11000000
	.db	%00000111,%11111111,%11000000
	.db	%00000111,%11111111,%11000000
	.db	%00000000,%00000000,%00000000
	.db	%00000000,%00000000,%00000000
	.db	%00000011,%11111111,%10000000
	.db	%00000111,%11111111,%11000000
	.db	%00000111,%11111111,%11000000
	.db	%00000111,%00000000,%00000000
	.db	%00000111,%00000000,%00000000
	.db	%00000111,%11111111,%00000000
	.db	%00000011,%11111111,%10000000
	.db	%00000001,%11111111,%11000000
	.db	%00000000,%00000001,%11000000
	.db	%00000000,%00000001,%11000000
	.db	%00000111,%11111111,%11000000
	.db	%00000111,%11111111,%11000000
	.db	%00000011,%11111111,%10000000
	.db	%00000000,%00000000,%00000000
	.db	%00000000,%00000000,%00000000
	.db	%00000011,%11111111,%10000000
	.db	%00000111,%11111111,%11000000
	.db	%00000111,%11111111,%11000000
	.db	%00000111,%00000000,%00000000
	.db	%00000111,%00000000,%00000000
	.db	%00000111,%11111111,%00000000
	.db	%00000011,%11111111,%10000000
	.db	%00000001,%11111111,%11000000
	.db	%00000000,%00000001,%11000000
	.db	%00000000,%00000001,%11000000
	.db	%00000111,%11111111,%11000000
	.db	%00000111,%11111111,%11000000
	.db	%00000011,%11111111,%10000000

#endif

winPlayers:
   .db "Players",0
#ifdef GUI_COMPILE
   .db 3
#endif
   .db "How many",0
   .db "players will",0
   .db "be playing?",0
   .db 2
   .db " One",0
   .db " Two",0
winDiff:
   .db "Difficulty",0
#ifdef GUI_COMPILE
   .db 3
#endif
   .db "Choose a",0
   .db "difficulty",0
   .db "level.",0
   .db 3
   .db " Easy",0
   .db " Med",0
   .db " Hard",0
winYouWin:
   .db "You Win!",0
#ifdef GUI_COMPILE
   .db 3
#endif
   .db "You have",0
   .db "won the game.",0
   .db "Try again?",0
   .db 2
   .db " Yes",0
   .db " No",0
winYouLose:
   .db "You Lose!",0
#ifdef GUI_COMPILE
   .db 3
#endif
   .db "You have",0
   .db "lost the game.",0
   .db "Try again?",0
   .db 2
   .db " Yes",0
   .db " No",0

WaitStr:
	.db "Waiting for other player...",0
SendStr:
	.db "Sending Board...",0
GetStr:
	.db "Recieving Board...",0

graysprite:
	.db %10101000
	.db %01010000
	.db %10101000
	.db %01010000
	.db %10101000
solidsprite:
	.db $F8,$F8,$F8,$F8,$F8
plussprite:
	.db $00,$20,$70,$20,$00

firstrotsprite:
	.db %01010000
	.db %10001000
	.db %00000000
	.db %10001000
	.db %01010000
secondrotsprite:
	.db %10101000
	.db %00000000
	.db %10001000
	.db %00000000
	.db %10101000